#!/usr/bin/perl
use strict;
use warnings;

# This script generates a ranking of systems based on the WMT10 segment-
# level human ranking task.  The system level rankings are calculated 
# as the percentage of times that a system's translations were judged 
# better than or equal to the translations of any other sytems in the 
# segment level ranking task.
# The script currently does not include "_ref" (sysid for reference 
# translation included in the WMT10 data) translations in the calculation of 
# rankings.
# Usage: perl wmt10_sysRnk.pl <human rank file>
# Input must be of the format used in the data_RNK_<langPair>.csv files 
# included in this distribution.
# E.g.:
# perl wmt10_sysRnk.pl data_RNK_Spanish-English.csv
# Output is written to tab-separated files separately for each language, 
# with the language pair indicated at the end of the file name and a .sysRnk 
# extension.
# The human system rankings produced by this script can be used as input for 
# Spearman correlation calculation with automatic metrics.

my $scoreFile = shift(@ARGV);

open SCORES, $scoreFile;

my %rankSum;
my %comps;
my %rank;

while (<SCORES>) {
# to avoid header lines, only match if 3rd item (srcindex) is numerical
    if (/(.*?),(.*?),(\d+),(.*?),(.*?),(.*?),(.*?),(.*?),(.*?),(.*?),(.*?),(.*?),(.*?),(.*?),(.*?),(.*?),(-?\d),(-?\d),(-?\d),(-?\d),(-?\d)/) {
	push (my @systems, $8, $10, $12, $14, $16);
	push (my @scores, $17, $18, $19, $20, $21);
	for (my $i=0; $i < @systems; $i++) {
	    for (my $j=0; $j < @systems; $j++) {
# ignore no-ranks, refs
		if ($scores[$i] != -1 && $scores[$j] != -1 && $systems[$i] ne "_ref" && $systems[$j] ne "_ref" && $systems[$i] ne $systems[$j]) {
		    my $sysA = $systems[$i];
		    my $scoreA = $scores[$i];
		    my $sysB = $systems[$j];
		    my $scoreB = $scores[$j];
		    $rankSum{$sysA}++ if $scoreA <= $scoreB;
		    $comps{$sysA}++;
		}
	    }
	}
    }
}
close SCORES;

# set ranksum to 0 for any sys that has comparisons but no ranksum (because it was always ranked lowest)
for my $sys (keys %comps) {
    $rankSum{$sys} = 0 unless $rankSum{$sys};
}

open SYSRNK_OUT, ">$scoreFile.sysRnk";

# by sys, get avg score by dividing ranksum by number of comparisons
for my $sys (keys %rankSum) {
    $rank{$sys} = $rankSum{$sys} / $comps{$sys} if $comps{$sys};
}

for my $sys (sort keys %rank) {
    print SYSRNK_OUT "newssyscombtest2010\t$sys\t$rank{$sys}\n";
}

close SYSRNK_OUT;

